@@ -1,5 +1,5 @@
 /*
-   $Id: Pg.h,v 1.4 2004/07/09 19:05:40 turnstep Exp $
+   $Id: Pg.h,v 1.6 2005/05/07 03:59:46 turnstep Exp $
 
    Copyright (c) 2000-2004 PostgreSQL Global Development Group
    Copyright (c) 1997,1998,1999,2000 Edmund Mergl
@@ -28,6 +28,11 @@ static int errno;
 #define INV_WRITE 0x00020000
 #endif
 
+/* For versions of DBI older than 1.41 */
+#ifndef DBIcf_Executed
+#define DBIcf_Executed 0x080000
+#endif
+
 #ifdef BUFSIZ
 #undef BUFSIZ
 #endif
@@ -41,6 +46,8 @@ static int errno;
 
 #include "dbdimp.h"		/* read in our implementation details */
 
+#include "large_object.h"
+
 #include <dbd_xsh.h>		/* installed by the DBI module	*/
 
 
